@@ -151,8 +151,8 @@ void run_and_check_transform_reduce_default(ViewType1 first_view,
   const auto r2 = KE::transform_reduce(
       "MYLABEL", ExecutionSpace(), KE::cbegin(first_view),
       KE::cbegin(first_view), KE::cbegin(second_view), init_value);
-  EXPECT_EQ(r1, init_value);
-  EXPECT_EQ(r2, init_value);
+  ASSERT_EQ(r1, init_value);
+  ASSERT_EQ(r2, init_value);
 
   // non-trivial cases
   const auto r3 = KE::transform_reduce(ExecutionSpace(), KE::cbegin(first_view),
@@ -168,10 +168,10 @@ void run_and_check_transform_reduce_default(ViewType1 first_view,
   const auto r6 = KE::transform_reduce("MYLABEL", ExecutionSpace(), first_view,
                                        second_view, init_value);
 
-  EXPECT_EQ(r3, result_value);
-  EXPECT_EQ(r4, result_value);
-  EXPECT_EQ(r5, result_value);
-  EXPECT_EQ(r6, result_value);
+  ASSERT_EQ(r3, result_value);
+  ASSERT_EQ(r4, result_value);
+  ASSERT_EQ(r5, result_value);
+  ASSERT_EQ(r6, result_value);
 }
 
 TEST_F(std_algorithms_numerics_test,
@@ -243,40 +243,37 @@ void run_and_check_transform_reduce_overloadA(ViewType1 first_view,
                                               ViewType2 second_view,
                                               ValueType init_value,
                                               ValueType result_value,
-                                              Args&&... args) {
+                                              Args const&... args) {
   // trivial cases
   const auto r1 = KE::transform_reduce(
       ExecutionSpace(), KE::cbegin(first_view), KE::cbegin(first_view),
-      KE::cbegin(second_view), init_value, std::forward<Args>(args)...);
+      KE::cbegin(second_view), init_value, args...);
 
-  const auto r2 =
-      KE::transform_reduce("MYLABEL", ExecutionSpace(), KE::cbegin(first_view),
-                           KE::cbegin(first_view), KE::cbegin(second_view),
-                           init_value, std::forward<Args>(args)...);
+  const auto r2 = KE::transform_reduce(
+      "MYLABEL", ExecutionSpace(), KE::cbegin(first_view),
+      KE::cbegin(first_view), KE::cbegin(second_view), init_value, args...);
 
-  EXPECT_EQ(r1, init_value);
-  EXPECT_EQ(r2, init_value);
+  ASSERT_EQ(r1, init_value);
+  ASSERT_EQ(r2, init_value);
 
   // non trivial cases
   const auto r3 = KE::transform_reduce(
       ExecutionSpace(), KE::cbegin(first_view), KE::cend(first_view),
-      KE::cbegin(second_view), init_value, std::forward<Args>(args)...);
+      KE::cbegin(second_view), init_value, args...);
 
   const auto r4 = KE::transform_reduce(
       "MYLABEL", ExecutionSpace(), KE::cbegin(first_view), KE::cend(first_view),
-      KE::cbegin(second_view), init_value, std::forward<Args>(args)...);
+      KE::cbegin(second_view), init_value, args...);
 
-  const auto r5 =
-      KE::transform_reduce(ExecutionSpace(), first_view, second_view,
-                           init_value, std::forward<Args>(args)...);
-  const auto r6 =
-      KE::transform_reduce("MYLABEL", ExecutionSpace(), first_view, second_view,
-                           init_value, std::forward<Args>(args)...);
+  const auto r5 = KE::transform_reduce(ExecutionSpace(), first_view,
+                                       second_view, init_value, args...);
+  const auto r6 = KE::transform_reduce("MYLABEL", ExecutionSpace(), first_view,
+                                       second_view, init_value, args...);
 
-  EXPECT_EQ(r3, result_value);
-  EXPECT_EQ(r4, result_value);
-  EXPECT_EQ(r5, result_value);
-  EXPECT_EQ(r6, result_value);
+  ASSERT_EQ(r3, result_value);
+  ASSERT_EQ(r4, result_value);
+  ASSERT_EQ(r5, result_value);
+  ASSERT_EQ(r6, result_value);
 }
 
 TEST_F(std_algorithms_numerics_test,
@@ -363,37 +360,35 @@ template <class ExecutionSpace, class ViewType, class ValueType, class... Args>
 void run_and_check_transform_reduce_overloadB(ViewType view,
                                               ValueType init_value,
                                               ValueType result_value,
-                                              Args&&... args) {
+                                              Args const&... args) {
   // trivial
-  const auto r1 =
-      KE::transform_reduce(ExecutionSpace(), KE::cbegin(view), KE::cbegin(view),
-                           init_value, std::forward<Args>(args)...);
+  const auto r1 = KE::transform_reduce(ExecutionSpace(), KE::cbegin(view),
+                                       KE::cbegin(view), init_value, args...);
 
-  const auto r2 = KE::transform_reduce("MYLABEL", ExecutionSpace(),
-                                       KE::cbegin(view), KE::cbegin(view),
-                                       init_value, std::forward<Args>(args)...);
+  const auto r2 =
+      KE::transform_reduce("MYLABEL", ExecutionSpace(), KE::cbegin(view),
+                           KE::cbegin(view), init_value, args...);
 
-  EXPECT_EQ(r1, init_value);
-  EXPECT_EQ(r2, init_value);
+  ASSERT_EQ(r1, init_value);
+  ASSERT_EQ(r2, init_value);
 
   // non trivial
-  const auto r3 =
-      KE::transform_reduce(ExecutionSpace(), KE::cbegin(view), KE::cend(view),
-                           init_value, std::forward<Args>(args)...);
+  const auto r3 = KE::transform_reduce(ExecutionSpace(), KE::cbegin(view),
+                                       KE::cend(view), init_value, args...);
 
-  const auto r4 = KE::transform_reduce("MYLABEL", ExecutionSpace(),
-                                       KE::cbegin(view), KE::cend(view),
-                                       init_value, std::forward<Args>(args)...);
-  const auto r5 = KE::transform_reduce(ExecutionSpace(), view, init_value,
-                                       std::forward<Args>(args)...);
+  const auto r4 =
+      KE::transform_reduce("MYLABEL", ExecutionSpace(), KE::cbegin(view),
+                           KE::cend(view), init_value, args...);
+  const auto r5 =
+      KE::transform_reduce(ExecutionSpace(), view, init_value, args...);
 
   const auto r6 = KE::transform_reduce("MYLABEL", ExecutionSpace(), view,
-                                       init_value, std::forward<Args>(args)...);
+                                       init_value, args...);
 
-  EXPECT_EQ(r3, result_value);
-  EXPECT_EQ(r4, result_value);
-  EXPECT_EQ(r5, result_value);
-  EXPECT_EQ(r6, result_value);
+  ASSERT_EQ(r3, result_value);
+  ASSERT_EQ(r4, result_value);
+  ASSERT_EQ(r5, result_value);
+  ASSERT_EQ(r6, result_value);
 }
 
 TEST_F(std_algorithms_numerics_test,
@@ -447,8 +442,8 @@ void run_and_check_reduce_overloadA(ViewType view, ValueType non_trivial_result,
       KE::reduce(ExecutionSpace(), KE::cbegin(view), KE::cbegin(view));
   const auto r2 = KE::reduce("MYLABEL", ExecutionSpace(), KE::cbegin(view),
                              KE::cbegin(view));
-  EXPECT_EQ(r1, trivial_result);
-  EXPECT_EQ(r2, trivial_result);
+  ASSERT_EQ(r1, trivial_result);
+  ASSERT_EQ(r2, trivial_result);
 
   // non trivial cases
   const auto r3 =
@@ -458,10 +453,10 @@ void run_and_check_reduce_overloadA(ViewType view, ValueType non_trivial_result,
   const auto r5 = KE::reduce(ExecutionSpace(), view);
   const auto r6 = KE::reduce("MYLABEL", ExecutionSpace(), view);
 
-  EXPECT_EQ(r3, non_trivial_result);
-  EXPECT_EQ(r4, non_trivial_result);
-  EXPECT_EQ(r5, non_trivial_result);
-  EXPECT_EQ(r6, non_trivial_result);
+  ASSERT_EQ(r3, non_trivial_result);
+  ASSERT_EQ(r4, non_trivial_result);
+  ASSERT_EQ(r5, non_trivial_result);
+  ASSERT_EQ(r6, non_trivial_result);
 }
 
 TEST_F(std_algorithms_numerics_test,
@@ -503,8 +498,8 @@ void run_and_check_reduce_overloadB(ViewType view, ValueType result_value,
                              KE::cbegin(view), init_value);
   const auto r2 = KE::reduce("MYLABEL", ExecutionSpace(), KE::cbegin(view),
                              KE::cbegin(view), init_value);
-  EXPECT_EQ(r1, init_value);
-  EXPECT_EQ(r2, init_value);
+  ASSERT_EQ(r1, init_value);
+  ASSERT_EQ(r2, init_value);
 
   // non trivial cases
   const auto r3 = KE::reduce(ExecutionSpace(), KE::cbegin(view), KE::cend(view),
@@ -514,10 +509,10 @@ void run_and_check_reduce_overloadB(ViewType view, ValueType result_value,
   const auto r5 = KE::reduce(ExecutionSpace(), view, init_value);
   const auto r6 = KE::reduce("MYLABEL", ExecutionSpace(), view, init_value);
 
-  EXPECT_EQ(r3, result_value);
-  EXPECT_EQ(r4, result_value);
-  EXPECT_EQ(r5, result_value);
-  EXPECT_EQ(r6, result_value);
+  ASSERT_EQ(r3, result_value);
+  ASSERT_EQ(r4, result_value);
+  ASSERT_EQ(r5, result_value);
+  ASSERT_EQ(r6, result_value);
 }
 
 TEST_F(std_algorithms_numerics_test,
@@ -553,8 +548,8 @@ void run_and_check_reduce_overloadC(ViewType view, ValueType result_value,
                              KE::cbegin(view), init_value, joiner);
   const auto r2 = KE::reduce("MYLABEL", ExecutionSpace(), KE::cbegin(view),
                              KE::cbegin(view), init_value, joiner);
-  EXPECT_EQ(r1, init_value);
-  EXPECT_EQ(r2, init_value);
+  ASSERT_EQ(r1, init_value);
+  ASSERT_EQ(r2, init_value);
 
   // non trivial cases
   const auto r3 = KE::reduce(ExecutionSpace(), KE::cbegin(view), KE::cend(view),
@@ -565,10 +560,10 @@ void run_and_check_reduce_overloadC(ViewType view, ValueType result_value,
   const auto r6 =
       KE::reduce("MYLABEL", ExecutionSpace(), view, init_value, joiner);
 
-  EXPECT_EQ(r3, result_value);
-  EXPECT_EQ(r4, result_value);
-  EXPECT_EQ(r5, result_value);
-  EXPECT_EQ(r6, result_value);
+  ASSERT_EQ(r3, result_value);
+  ASSERT_EQ(r4, result_value);
+  ASSERT_EQ(r5, result_value);
+  ASSERT_EQ(r6, result_value);
 }
 
 TEST_F(std_algorithms_numerics_test,
