@@ -21,12 +21,15 @@
 #include <Kokkos_Core.hpp>
 #include <HIP/Kokkos_HIP.hpp>
 #include <HIP/Kokkos_HIP_Instance.hpp>
+#include <HIP/Kokkos_HIP_IsXnack.hpp>
 
 #include <impl/Kokkos_DeviceManagement.hpp>
 #include <impl/Kokkos_ExecSpaceManager.hpp>
 
 #include <hip/hip_runtime_api.h>
 
+#include <iostream>
+
 namespace Kokkos {
 
 #ifdef KOKKOS_ENABLE_DEPRECATED_CODE_4
@@ -46,50 +49,67 @@ void HIP::impl_initialize(InitializationSettings const& settings) {
   const int hip_device_id =
       Impl::get_gpu(settings).value_or(visible_devices[0]);
 
-  Impl::HIPInternal::m_hipDev = hip_device_id;
   KOKKOS_IMPL_HIP_SAFE_CALL(
       hipGetDeviceProperties(&Impl::HIPInternal::m_deviceProp, hip_device_id));
-  const auto& hipProp = Impl::HIPInternal::m_deviceProp;
   KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(hip_device_id));
 
-  // number of multiprocessors
-  Impl::HIPInternal::m_multiProcCount = hipProp.multiProcessorCount;
-
-  //----------------------------------
-  // Maximum number of warps,
-  // at most one warp per thread in a warp for reduction.
-  Impl::HIPInternal::m_maxWarpCount =
-      hipProp.maxThreadsPerBlock / Impl::HIPTraits::WarpSize;
-  if (Impl::HIPTraits::WarpSize < Impl::HIPInternal::m_maxWarpCount) {
-    Impl::HIPInternal::m_maxWarpCount = Impl::HIPTraits::WarpSize;
+  // Check that we are running on the expected architecture. We print a warning
+  // instead of erroring out because AMD does not guarantee that gcnArchName
+  // will always contain the gfx flag.
+  if (Kokkos::show_warnings()) {
+    if (std::string_view arch_name =
+            Impl::HIPInternal::m_deviceProp.gcnArchName;
+        arch_name.find(KOKKOS_ARCH_AMD_GPU) != 0) {
+      std::cerr
+          << "Kokkos::HIP::initialize WARNING: running kernels compiled for "
+          << KOKKOS_ARCH_AMD_GPU << " on " << arch_name << " device.\n";
+    }
   }
 
-  //----------------------------------
-  // Maximum number of blocks
-  Impl::HIPInternal::m_maxBlock[0] = hipProp.maxGridSize[0];
-  Impl::HIPInternal::m_maxBlock[1] = hipProp.maxGridSize[1];
-  Impl::HIPInternal::m_maxBlock[2] = hipProp.maxGridSize[2];
+  // Print a warning if the user did not select the right GFX942 architecture
+#ifdef KOKKOS_ARCH_AMD_GFX942
+  if ((Kokkos::show_warnings()) &&
+      (Impl::HIPInternal::m_deviceProp.integrated == 1)) {
+    std::cerr << "Kokkos::HIP::initialize WARNING: running kernels for MI300X "
+                 "(discrete GPU) on a MI300A (APU).\n";
+  }
+#endif
+#ifdef KOKKOS_ARCH_AMD_GFX942_APU
+  if (!Kokkos::Impl::xnack_environment_enabled()) {
+    std::cerr << R"warning(
+Kokkos::HIP::initialize WARNING: Could not determine that xnack is enabled.
+                                 Kokkos requires xnack to be enabled for
+                                 ARCH_AMD_GFX942_APU (MI300A) to access host
+                                 allocations from the device. Set HSA_XNACK=1
+                                 in your environment. For further information
+                                 on HMM support call `Kokkos::print_configuration`,
+                                 or run with KOKKOS_PRINT_CONFIGURATION=1 in your
+                                 environment.
+)warning";
+  }
 
-  // theoretically, we can get 40 WF's / CU, but only can sustain 32 see
-  // https://github.com/ROCm-Developer-Tools/HIP/blob/a0b5dfd625d99af7e288629747b40dd057183173/vdi/hip_platform.cpp#L742
-  Impl::HIPInternal::m_maxWavesPerCU = 32;
-  Impl::HIPInternal::m_shmemPerSM    = hipProp.maxSharedMemoryPerMultiProcessor;
-  Impl::HIPInternal::m_maxShmemPerBlock = hipProp.sharedMemPerBlock;
+  if ((Kokkos::show_warnings()) &&
+      (Impl::HIPInternal::m_deviceProp.integrated == 0)) {
+    std::cerr << "Kokkos::HIP::initialize WARNING: running kernels for MI300A "
+                 "(APU) on a MI300X (discrete GPU).\n";
+  }
+#endif
+
+  // theoretically on GFX 9XX GPUs, we can get 40 WF's / CU, but only can
+  // sustain 32 see
+  // https://github.com/ROCm/clr/blob/4d0b815d06751735e6a50fa46e913fdf85f751f0/hipamd/src/hip_platform.cpp#L362-L366
+  const int maxWavesPerCU =
+      Impl::HIPInternal::m_deviceProp.major <= 9 ? 32 : 64;
   Impl::HIPInternal::m_maxThreadsPerSM =
-      Impl::HIPInternal::m_maxWavesPerCU * Impl::HIPTraits::WarpSize;
+      maxWavesPerCU * Impl::HIPTraits::WarpSize;
 
   // Init the array for used for arbitrarily sized atomics
   desul::Impl::init_lock_arrays();  // FIXME
 
-  // Allocate a staging buffer for constant mem in pinned host memory
-  // and an event to avoid overwriting driver for previous kernel launches
-  KOKKOS_IMPL_HIP_SAFE_CALL(
-      hipHostMalloc((void**)&Impl::HIPInternal::constantMemHostStaging,
-                    Impl::HIPTraits::ConstantMemoryUsage));
-
-  KOKKOS_IMPL_HIP_SAFE_CALL(
-      hipEventCreate(&Impl::HIPInternal::constantMemReusable));
+  // Set singleton device id
+  Impl::HIPInternal::singleton().m_hipDev = hip_device_id;
 
+  // Create the singleton stream and initialize singleton instance.
   hipStream_t singleton_stream;
   KOKKOS_IMPL_HIP_SAFE_CALL(hipStreamCreate(&singleton_stream));
   Impl::HIPInternal::singleton().initialize(singleton_stream);
@@ -100,12 +120,17 @@ void HIP::impl_finalize() {
 
   desul::Impl::finalize_lock_arrays();  // FIXME
 
-  KOKKOS_IMPL_HIP_SAFE_CALL(
-      hipEventDestroy(Impl::HIPInternal::constantMemReusable));
-  KOKKOS_IMPL_HIP_SAFE_CALL(
-      hipHostFree(Impl::HIPInternal::constantMemHostStaging));
+  for (const auto hip_device : Impl::HIPInternal::hip_devices) {
+    KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(hip_device));
+    KOKKOS_IMPL_HIP_SAFE_CALL(
+        hipEventDestroy(Impl::HIPInternal::constantMemReusable[hip_device]));
+    KOKKOS_IMPL_HIP_SAFE_CALL(
+        hipHostFree(Impl::HIPInternal::constantMemHostStaging[hip_device]));
+  }
 
   Impl::HIPInternal::singleton().finalize();
+  KOKKOS_IMPL_HIP_SAFE_CALL(
+      hipSetDevice(Impl::HIPInternal::singleton().m_hipDev));
   KOKKOS_IMPL_HIP_SAFE_CALL(
       hipStreamDestroy(Impl::HIPInternal::singleton().m_stream));
 }
@@ -146,12 +171,13 @@ void HIP::print_configuration(std::ostream& os, bool /*verbose*/) const {
 #else
   os << "no\n";
 #endif
-#ifdef KOKKOS_ENABLE_IMPL_HIP_UNIFIED_MEMORY
-  os << "  KOKKOS_ENABLE_IMPL_HIP_UNIFIED_MEMORY: ";
-  os << "yes\n";
-#endif
 
   os << "\nRuntime Configuration:\n";
+  os << "  XNACK environment variable set: ";
+  os << (Kokkos::Impl::xnack_environment_enabled() ? "yes\n" : "no\n");
+  os << "  Kernel reports HMM module via `CONFIG_HMM_MIRROR=y` in "
+        "`/boot/config`: ";
+  os << (Kokkos::Impl::xnack_boot_config_has_hmm_mirror() ? "yes\n" : "no\n");
 
   m_space_instance->print_configuration(os);
 }
@@ -164,7 +190,12 @@ void HIP::impl_static_fence(const std::string& name) {
       name,
       Kokkos::Tools::Experimental::SpecialSynchronizationCases::
           GlobalDeviceSynchronization,
-      [&]() { KOKKOS_IMPL_HIP_SAFE_CALL(hipDeviceSynchronize()); });
+      [&]() {
+        for (const auto hip_device : Impl::HIPInternal::hip_devices) {
+          KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(hip_device));
+          KOKKOS_IMPL_HIP_SAFE_CALL(hipDeviceSynchronize());
+        }
+      });
 }
 
 void HIP::fence(const std::string& name) const {
